"""Core processing pipeline and parameter optimization components."""

from flamingo.core.processing_pipeline import ProcessingPipeline
from flamingo.core.optimization_parameter import CorrectionConfig